<?php

/**
 * @file
 * Contains functions for the integration of e-Commerce module with pay-per-node module.
 */

/*
REM - ECOMMERCE 4 HOOKS

paypernode_product_form                       called at form rendering
paypernode_product_validate                   called at form validation
paypernode_product_insert                     called at inserting product
paypernode_product_delete                     called at product delete
paypernode_product_update                     called at editing form
paypernode_product_load                       called at product load (form, view)
paypernode_product_attributes                 called at product view (and after paypernode_product_adjust_price)
paypernode_product_adjust_price called before showing price
paypernode_product_ec_checkout_validate_item  called to invalidate the cart modification
paypernode_product_ec_checkout_add_complete   called when adding product to cart is completye
paypernode_product_product_checkout_init
paypernode_product_product_calculate
paypernode_product_cart_remove_item
paypernode_product_transaction_insert
paypernode_product_txn_allocation             called on payment status change
paypernode_product_txn_workflow               called on workflow status change
paypernode_product_transaction_load
paypernode_product_checkout_post_submit
paypernode_product_transaction_update
*/


/*
implementation of hook_product_form()
*/
function paypernode_product_form(&$node, $op, $a3 = NULL, $a4 = NULL) {
  $form['node_qtty'] = array(
      '#type' => 'textfield',
      '#title' => t('Quantity'),
      '#default_value' => isset($node->node_qtty) ? $node->node_qtty : 1,
      '#description' => t("How many nodes the user will be allowed to create."),
  );
  $form['node_type'] = array(
    '#type' => 'radios',
    '#title' => t('Node type'),
    '#default_value' => $node->node_type,
    '#options' => paypernode_get_types(),
    '#description' => t("Node type the user will be allowed to create."),
  );
  return $form;
}

/*
implementation of hook_product_txn_allocation()
*/
function paypernode_product_txn_allocation(&$node, $op, $a3 = NULL, $a4 = NULL) {
  // if the payment is completed
  if ($op->allocation==RECEIPT_STATUS_COMPLETED) {
    $paypernode=db_fetch_object(db_query('SELECT * FROM {ec_product_paypernode} WHERE vid = %d', $node->vid));
    
    if ($paypernode->node_type && $node->qty && $paypernode->node_qtty) {
      $number = $node->qty * $paypernode->node_qtty;
      $uid = db_result(db_query('SELECT ecid FROM {ec_transaction} WHERE txnid = %d', $node->txnid));
      
      // anonymous not allowed
      if ($uid>0) {
        paypernode_user_update($uid, $paypernode->node_type, $number);
      }
      else {
        // Error condition
        watchdog('paypernode', "Cannot activate %productid for an anomymous user: transaction id %txnid.", array('%productid' => $node->nid, '%txnid' => $node->txnid), WATCHDOG_ERROR);
      }
    
    } 
    else {
      // Error condition
      watchdog('paypernode', "No new nodes types added for product %productid.", array('%productid' => $node->nid), WATCHDOG_ERROR);
    }
  }
 

}

/*
implementation of hook_product_validate()
*/
function paypernode_product_validate(&$node, $op, $a3 = NULL, $a4 = NULL) {
  return TRUE;
}

/*
implementation of hook_product_insert()
*/
function paypernode_product_insert(&$node, $op, $a3 = NULL, $a4 = NULL) {
  return db_query("INSERT INTO {ec_product_paypernode} (nid, vid, node_type, node_qtty) VALUES ('%d', '%d', '%s', '%d')", $node->nid, $node->vid, $node->node_type, $node->node_qtty);
}

/*
implementation of hook_product_update()
*/
function paypernode_product_update(&$node, $op, $a3 = NULL, $a4 = NULL) {
  return db_query("UPDATE {ec_product_paypernode} SET node_type = '%s', node_qtty ='%d' WHERE vid = '%d'", $node->node_type, $node->node_qtty, $node->vid);
}

/*
implementation of hook_product_load()
*/
function paypernode_product_load(&$node, $op, $a3 = NULL, $a4 = NULL) {
  return db_fetch_object(db_query('SELECT * FROM {ec_product_paypernode} WHERE vid = %d', $node->vid));
}

/*
implementation of hook_product_delete()
*/
function paypernode_product_delete(&$node, $op, $a3 = NULL, $a4 = NULL) {
  return db_query('DELETE FROM {ec_product_paypernode} WHERE nid = %d', $node->nid);
}

/*
function paypernode_productapi(&$node, $op, $a3 = null, $a4 = null) {
  switch ($op) {
    case 'fields':
      return array('node_type' => $node->node_type);
    case 'attributes':
      return array('in_stock');
    case 'cart add item':
      break;
    case 'delete':
      // Note: cannot delete single revisions
      return db_query('DELETE FROM {ec_product_paypernode} WHERE nid = %d', $node->nid);
  }
}
*/


// TODO RECURRING PAYMENT HANDLING
/**
 * Implementation of hook_recurringapi().
 *
function paypernode_recurringapi($op, &$obj, $val1 = NULL, $val2 = NULL) {
  switch ($op) {
    case 'on expiry':
      paypernode_atexpiry($obj);
      break;
    case 'on purchase':
      // If it's a renewal then the node is already created, and thus we need
      // to decrement the number they are allowed to create
      if ($val2) {
        $node = node_load($val1);
        db_query("UPDATE {ec_paypernode_user} SET allowed = allowed - %d WHERE uid = %d AND node_type = '%s'", $node->node_qtty, $obj['uid'], $node->node_type);
      }
      break;
    case 'cron report':
      print "Nodes removed: ".$GLOBALS['expirations']['nodes_removed']."\n\n";
      break;
  }
}
*/


// TODO RECURRING PAYMENT HANDLING
/**
 * Process node removals at expiry time and unpublish the nodes from
 * the system.
 *
 * This function increments $GLOBALS['expirations']['nodes_removed'] each time
 * it removes a node.
 *
 * @param $node Product for which nodes are processed. This function expects
 * $node->expired_schedule to be set to the expiring entry from
 * ec_recurring_schedule and $node->uid to be the user that purchased the
 * product.
 *
function paypernode_atexpiry($node) {
  global $expirations;

  // Do nothing if the product was previously renewed
  if ($node->expired_schedule['status'] == ECRECURRING_STATUS_RENEWED) {
    return;
  }

  // Try and find the node(s) that needs to be unpublished.
  // NOTE: This would probably be a lot better if we could somehow associate pay-per-nodes with the transaction they were purchased from, but that would require some big changes.
  $qty = db_fetch_object(db_query("SELECT qty FROM {ec_transaction_product} WHERE txnid = %d AND nid = %d", $node->txnid, $node->nid));
  $result = db_query("SELECT n.nid FROM {node} n WHERE n.type = '%s' AND n.uid = %d AND n.status = 1 ORDER BY n.created LIMIT {$qty->qty}", $node->node_type, $node->expired_schedule['uid']);
  $nodes = array();
  while ($nid = db_fetch_object($result)) {
    $nodes[] = $nid->nid;
    $expirations['nodes_removed']++;
  }
  node_operations_unpublish($nodes);
  // Also make them anonymous so the user can't re-publish them
  db_query('UPDATE {node} SET uid = 0 WHERE nid IN(%s)', implode(',', $nodes));
}
*/
